package net.ibizsys.central.plugin.boot.core.runtime;

import net.ibizsys.central.SystemGateway;
import net.ibizsys.central.database.ISysDBSchemeRuntime;
import net.ibizsys.central.database.SysDBSchemeRuntime;
import net.ibizsys.central.dataentity.DataEntityRuntime;
import net.ibizsys.central.dataentity.IDataEntityRuntime;
import net.ibizsys.central.dataentity.service.IDEService;
import net.ibizsys.model.IPSSystemService;
import net.ibizsys.model.PSModelServiceImpl;
import net.ibizsys.model.dataentity.IPSDataEntity;
import org.springframework.context.annotation.Primary;

import java.lang.annotation.Annotation;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class BootSystemRuntime extends net.ibizsys.central.cloud.core.ServiceSystemRuntime implements IBootSystemRuntime {

	private static IBootSystemRuntime instance = null;

    public static IBootSystemRuntime getInstance() {
        return getInstance(false);
    }

    private static Map<String, IBootService> localDEServiceMap = new ConcurrentHashMap();

    public static synchronized void addLocalDEServiceMap(String dataEntityid,IBootService service)
    {
        if(localDEServiceMap.containsKey(dataEntityid)&&service.getBaseService().getClass().getAnnotation(Primary.class)==null)
            return;
        localDEServiceMap.put(dataEntityid,service);
    }

    public static IBootSystemRuntime getInstance(boolean bTryMode) {
        if(BootSystemRuntime.instance != null || bTryMode) {
            return BootSystemRuntime.instance;
        }
        throw new RuntimeException("核心系统运行时实例不存在");
    }


    public static void setInstance(IBootSystemRuntime instance) {
        BootSystemRuntime.instance = instance;
    }

    @Override
    protected void onStart() throws Exception {
        super.onStart();
        if(getInstance(true) == null) {
            setInstance(this);
        }
    }

    @Override
    public IDEService getDEService(IDataEntityRuntime iDataEntityRuntime) {
        IBootService service= localDEServiceMap.get(iDataEntityRuntime.getId());
        if(service==null)
            return super.getDEService(iDataEntityRuntime);
        if(service.getSystemRuntime()==null)
            service.setSystemRuntime(this);
        return service;
    }

    public boolean isEnableDEService(IPSDataEntity iPSDataEntity) {
        return this.localDEServiceMap.containsKey(iPSDataEntity.getId());
    }

    
    
    @Override
    protected IDataEntityRuntime createDefaultDataEntityRuntime() {
    	return new DataEntityRuntime();
    }
    
    
    @Override
	protected ISysDBSchemeRuntime createDefaultSysDBSchemeRuntime() {
		return new SysDBSchemeRuntime();
	}

    @Override
    public String getName() {
        return "iBizBoot";
    }


    @Override
    public void registerDEServiceObj(String strDataEntityId, String strDEServiceObj) {
        super.registerDEServiceObj(strDataEntityId,strDEServiceObj);
    }
}
